package net.wanji.datacenter.kafka;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.datacenter.constant.EventAbnormalEnum;
import net.wanji.datacenter.pojo.dto.CrossEventDTO;
import net.wanji.datacenter.pojo.dto.CrossLaneSnapshotDataDTO;
import net.wanji.datacenter.pojo.dto.CrossSnapshotDataDTO;
import net.wanji.datacenter.pojo.dto.LineCongestion;
import net.wanji.datacenter.pojo.po.PhaseEmptyResult;
import net.wanji.datacenter.service.DataProcessService;
import net.wanji.datacenter.util.RedisUtils;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * @author duanruiming
 * @date 2023/03/09 16:03
 */
@Component
@Slf4j
public class ConsumerHandler implements KafkaListenerErrorHandler {

    // Redis缓存过期时间
    public static final int TTL_IN_MILLIS = 10 * 60 * 1000;
    @Resource(name = "lanePeriodicDataProcessService")
    DataProcessService lanePeriodicDataProcessService;
    @Resource(name = "crossSnapshotDataProcessService")
    DataProcessService crossSnapshotDataProcessService;
    @Resource(name = "laneSnapshotDataProcessService")
    DataProcessService laneSnapshotDataProcessService;

    @Autowired
    RedisUtils redisUtils;

    public static final Map<String, List<CrossSnapshotDataDTO>> crossSnapshotDataMap = new ConcurrentHashMap<>();

    public static final List<PhaseEmptyResult> phaseEmptyResultList = new CopyOnWriteArrayList<>();


    @KafkaListener(topics = {"${kafka-consumer.crossEventIndex}"}, groupId = "group")
    public void receiveCrossEventIndex(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        // 同一时间，可能发送两条数据，东西方向，转向可以忽略
        PhaseEmptyResult current = JacksonUtils.getInstance().readValue((String) record.value(), PhaseEmptyResult.class);
        if (!CollectionUtils.isEmpty(phaseEmptyResultList)) {
            for (PhaseEmptyResult item : phaseEmptyResultList) {
                String crossId = item.getCrossId();
                Long startTime = item.getStartTime();
                String direction = item.getDirection();
                if (Objects.equals(crossId, current.getCrossId()) && Objects.equals(direction, current.getDirection())
                        && current.getStartTime() - startTime < 1000 * 60 * 5) {
                    phaseEmptyResultList.add(current);
                    phaseEmptyResultList.remove(item);
                }
            }
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.lanePeriodicDataTopic}"}, groupId = "group")
    public void receiveLanePeriodicData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        Object convert = lanePeriodicDataProcessService.convert(String.valueOf(record.value()));
        // 修改逻辑，将保存逻辑改为将方向，转向数据返回kafka
        lanePeriodicDataProcessService.save(convert);
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.crossSnapshotDataTopic}"}, groupId = "group")
    public void receiveCrossSnapshotData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        CrossSnapshotDataDTO crossSnapshotDataDTO = (CrossSnapshotDataDTO) crossSnapshotDataProcessService.convert(String.valueOf(record.value()));
        if (!crossSnapshotDataMap.isEmpty() && !CollectionUtils.isEmpty(crossSnapshotDataMap.get(record.topic()))) {
            List<CrossSnapshotDataDTO> crossSnapshotDataDTOS = crossSnapshotDataMap.get(record.topic());
            crossSnapshotDataDTOS.add(crossSnapshotDataDTO);
            if (crossSnapshotDataDTOS.size() == 30) {
                crossSnapshotDataDTOS.remove(crossSnapshotDataDTOS.get(0));
            }
        } else {
            List<CrossSnapshotDataDTO> crossSnapshotDataDTOS = new ArrayList<>(30);
            crossSnapshotDataDTOS.add(crossSnapshotDataDTO);
            crossSnapshotDataMap.put(record.topic(), crossSnapshotDataDTOS);
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.laneSnapshotDataTopic}"}, groupId = "group")
    public void receiveLaneSnapshotData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            List<CrossLaneSnapshotDataDTO> convertList = (List<CrossLaneSnapshotDataDTO>) laneSnapshotDataProcessService.convert(String.valueOf(record.value()));
            if (!CollectionUtils.isEmpty(convertList)) {
                laneSnapshotDataProcessService.save(convertList);
            }
        } catch (Exception e) {
            log.error("Kafka收到车道快照数据异常", e);
            throw new Exception(e);
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"cross.event.index"}, groupId = "cross-event-index")
    public void receiveCrossEventData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment)
            throws Exception {

        try {
            ObjectMapper objectMapper = new ObjectMapper();

            String recordStr = String.valueOf(record.value());
            CrossEventDTO crossEventDTO = JacksonUtils.getInstance().readValue(recordStr, CrossEventDTO.class);
            String msgType = crossEventDTO.getMsgType();
            String crossId = crossEventDTO.getCrossId();

            if (Objects.equals(msgType, EventAbnormalEnum.CROSS_UNBALANCE.getType())) {
                String redisListElement = objectMapper.writeValueAsString(crossEventDTO);
                // 失衡事件
                // 方向缓存
                Integer unbalanceDir = crossEventDTO.getUnbalanceDir();
                String redisKey = crossId + ":" + unbalanceDir;
                redisUtils.addToSortedSetWithExpiry(redisKey, redisListElement, TTL_IN_MILLIS);
                // 路口缓存
                redisUtils.addToSortedSetWithExpiry(crossId, redisListElement, TTL_IN_MILLIS);
            } else if (msgType.startsWith("50")) {
                // 拥堵事件
                // 方向缓存
                List<CrossEventDTO> details = crossEventDTO.getDetails();
                for (CrossEventDTO detail : details) {
                    detail.setMsgType(detail.getCongestionCode());
                    Integer dir = detail.getDir();
                    String redisKey = crossId + ":" + dir;
                    String redisListElement = objectMapper.writeValueAsString(detail);
                    redisUtils.addToSortedSetWithExpiry(redisKey, redisListElement, TTL_IN_MILLIS);
                }
                // 路口缓存
                String redisListElement = objectMapper.writeValueAsString(crossEventDTO);
                redisUtils.addToSortedSetWithExpiry(crossId, redisListElement, TTL_IN_MILLIS);
            } else if (Objects.equals(msgType, EventAbnormalEnum.CROSS_OVERFLOW.getType())
                    || Objects.equals(msgType, EventAbnormalEnum.CROSS_DEADLOCK.getType())) {
                // 溢出和死锁事件
                // 方向缓存
                List<CrossEventDTO> details = crossEventDTO.getDetails();
                for (CrossEventDTO detail : details) {
                    detail.setMsgType(EventAbnormalEnum.CROSS_OVERFLOW.getType());
                    Integer dir = detail.getDir();
                    String redisKey = crossId + ":" + dir;
                    String redisListElement = objectMapper.writeValueAsString(detail);
                    redisUtils.addToSortedSetWithExpiry(redisKey, redisListElement, TTL_IN_MILLIS);
                }
                // 路口缓存
                String redisListElement = objectMapper.writeValueAsString(crossEventDTO);
                redisUtils.addToSortedSetWithExpiry(crossId, redisListElement, TTL_IN_MILLIS);
            }
        } catch (Exception e) {
            log.error("Kafka收到路口事件数据异常", e);
            throw new Exception(e);
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"line_congestion_sink"}, groupId = "line_congestion_sink_redis_consumer")
    public void receiveLineCongestionData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment)
            throws Exception {

        try {
            ObjectMapper objectMapper = new ObjectMapper();

            String recordStr = String.valueOf(record.value());
            //LineCongestion dto = JacksonUtils.getInstance().readValue(recordStr, LineCongestion.class);
            //LineCongestion dto = JSONObject.parseObject(recordStr, LineCongestion.class);//objectMapper.readValue(element.toString(), LineCongestion.class);

            String redisElement = recordStr; //JSONObject.toJSONString(dto, SerializerFeature.WriteNonStringValueAsString); //objectMapper.writeValueAsString(dto);

            String redisKey = "line_congestion_sink";

            log.info("消费干线指标信息：{}",redisElement);
            redisUtils.addToSortedSetWithExpiry(redisKey, redisElement, TTL_IN_MILLIS);
        } catch (Exception e) {
            log.error("Kafka收到路口事件数据异常", e);
            throw new Exception(e);
        }
        acknowledgment.acknowledge();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

}
