package net.wanji.datacenter.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.datacenter.cache.*;
import net.wanji.datacenter.common.Constants;
import net.wanji.datacenter.constant.Constant;
import net.wanji.datacenter.constant.EventAbnormalEnum;
import net.wanji.datacenter.kafka.ConsumerHandler;
import net.wanji.datacenter.kafka.ProducerHandler;
import net.wanji.datacenter.pojo.convert.LanePeriodicDataEventListConvert;
import net.wanji.datacenter.pojo.dto.CrossDataIndexDTO;
import net.wanji.datacenter.pojo.dto.CrossEventDTO;
import net.wanji.datacenter.pojo.dto.LanePeriodicDataDTO;
import net.wanji.datacenter.pojo.dto.LineCongestion;
import net.wanji.datacenter.pojo.po.PhaseEmptyResult;
import net.wanji.datacenter.service.DataProcessService;
import net.wanji.datacenter.util.RedisUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.text.DecimalFormat;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/10 10:30
 * @description: 车道周期实时数据加工处理服务，优化模块实时监控操作数据
 */
@Slf4j
@Service(value = "lanePeriodicDataProcessService")
@RequiredArgsConstructor
@SuppressWarnings("all")
public class LanePeriodicDataProcessServiceImpl implements DataProcessService {

    private final ConsumerHandler consumerHandler;
    private final CrossBaseLaneInfoMapper crossBaseLaneInfoMapper;
    private static CrossDirDataRealtimeMapper crossDirDataRealTimeMapper;
    private static CrossTurnDataRealtimeMapper crossTurnDataRealTimeMapper;
    private final CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    private static CrossDataRealtimeMapper crossDataRealTimeMapper;
    private final LanePeriodicDataEventListConvert lanePeriodicDataEventListConvert;
    private final ProducerHandler producerHandler;
    private static GreenwaveHistMapper greenwaveHistMapper;
    private static GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    private static GreenwaveInfoMapper greenwaveInfoMapper;
    // 五分钟周期内是否有全场红
    public static final ConcurrentHashMap<String, Integer> periodHaveAllRedMap = new ConcurrentHashMap<>();

    private static final Map<String, String> pedestrianMap = new HashMap<String, String>();

    static {
        pedestrianMap.put("11", "13NF80B5QN0");
        pedestrianMap.put("12", "13NF80B5QN0");
        pedestrianMap.put("13", "13NF80B5QN0");
        pedestrianMap.put("14", "13NF80B5QN0");
        pedestrianMap.put("21", "13NGH0B5RC0");
        pedestrianMap.put("22", "13NGH0B5RC0");
        pedestrianMap.put("23", "13NGH0B5RC0");
        pedestrianMap.put("24", "13NGH0B5RC0");
        pedestrianMap.put("31", "13NI00B5RM0");
        pedestrianMap.put("32", "13NI00B5RM0");
    }

    @Autowired
    RedisUtils redisUtils;

    @Autowired
    public void setCrossDataRealTimeMapper(CrossDataRealtimeMapper crossDataRealTimeMapper,
                                           CrossDirDataRealtimeMapper crossDirDataRealTimeMapper,
                                           CrossTurnDataRealtimeMapper crossTurnDataRealTimeMapper,
                                           GreenwaveHistMapper greenwaveHistMapper,
                                           GreenwaveRealtimeMapper greenwaveRealtimeMapper,
                                           GreenwaveInfoMapper greenwaveInfoMapper) {
        this.crossDataRealTimeMapper = crossDataRealTimeMapper;
        this.crossDirDataRealTimeMapper = crossDirDataRealTimeMapper;
        this.crossTurnDataRealTimeMapper = crossTurnDataRealTimeMapper;
        this.greenwaveHistMapper = greenwaveHistMapper;
        this.greenwaveRealtimeMapper = greenwaveRealtimeMapper;
        this.greenwaveInfoMapper = greenwaveInfoMapper;
    }


    @Override
    public void send(String topic, Object convertData) {
        producerHandler.send2FirstKafka(topic, convertData);
        producerHandler.send2SecondKafka(topic, convertData);
    }

    @Override
    public Object convert(String originalData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataPO = null;
        try {
            lanePeriodicDataPO = JacksonUtils.getInstance().readValue(originalData, LanePeriodicDataDTO.class);
        } catch (Exception e) {
            log.error("车道周期实时数据转换异常", e);
            throw new Exception();
        }
        return lanePeriodicDataPO;
    }

    @Override
    @Transactional
    public void save(Object convertData) throws Exception {
        try {
            LanePeriodicDataDTO lanePeriodicDataDTO = (LanePeriodicDataDTO) convertData;

            int laneNum = lanePeriodicDataDTO.getLaneNum();
            List<CrossDataRealtimePO> crossDataRealtimePOS = new ArrayList<>(laneNum);
            List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = new ArrayList<>(laneNum);
            List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = new ArrayList<>(laneNum);
            List<CrossLaneDataRealTimePO> crossLaneDataRealTimePOS = new ArrayList<>(laneNum);

            List<LanePeriodicDataDTO.EventList> eventLists = lanePeriodicDataDTO.getEventList();
            String timeStamp = lanePeriodicDataDTO.getTimeStamp();
            Date endDate = DateUtil.parse(timeStamp, DateStyle.YYYY_MM_DD_HH_MM_SS.getValue());
            Date startDate = DateUtil.offsetMinute(endDate, -5);

            if (CollectionUtils.isEmpty(eventLists)) {
                log.error("当前车道周期实时数据车道列表为空,参数信息：", lanePeriodicDataDTO);
                throw new Exception("当前车道周期实时数据车道列表为空");
            }
            // 通过车道编号获取车道信息
            List<String> LaneIdList = eventLists.stream()
                    .map(LanePeriodicDataDTO.EventList::getLaneId)
                    .collect(Collectors.toList());
            List<CrossBaseLaneInfoPO> crossBaseLaneInfoPOS = crossBaseLaneInfoMapper.selectBatchIds(LaneIdList);

            DecimalFormat decimalFormat = new DecimalFormat("#.##");
            // 将kafka车道数据与路口车道数据对应，获取路口编号，方向，转向，分组计算
            for (LanePeriodicDataDTO.EventList laneRealTimeItem : eventLists) {
                String laneId = laneRealTimeItem.getLaneId();

                if (laneId.length() == 2) {
                    CrossBaseLaneInfoPO laneInfoPO = new CrossBaseLaneInfoPO();
                    laneInfoPO.setCrossId(pedestrianMap.get(laneId));
                    laneInfoPO.setId(String.join(Constant.SEPARATOR_MINUS, pedestrianMap.get(laneId), laneId));
                    crossLaneDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossLaneDataRealTimePO(laneRealTimeItem, laneInfoPO, startDate));
                }
                for (CrossBaseLaneInfoPO laneInfoPO : crossBaseLaneInfoPOS) {
                    String id = laneInfoPO.getId();
                    if (StringUtils.equals(laneId, id) && Objects.equals(laneInfoPO.getType(), 2)) {

                        crossDataRealtimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDataRealTimePO(laneRealTimeItem, laneInfoPO));

                        crossDirDataRealtimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDirDataRealTimePO(laneRealTimeItem, laneInfoPO));

                        crossTurnDataRealtimePOS.add(lanePeriodicDataEventListConvert.convert2CrossTurnDataRealtimePO(laneRealTimeItem, laneInfoPO));

                        crossLaneDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossLaneDataRealTimePO(laneRealTimeItem, laneInfoPO, startDate));
                    }
                }
            }
            Map<String, List<CrossLaneDataRealTimePO>> laneDataMap = new HashMap<>();
            laneDataMap.put(Constant.REAL_TIME_LANE_DATA_KEY, crossLaneDataRealTimePOS);
            producerHandler.send2FirstKafka(Constant.REAL_TIME_LANE_DATA_TOPIC, laneDataMap);
            producerHandler.send2SecondKafka(Constant.REAL_TIME_LANE_DATA_TOPIC, laneDataMap);

            List<CrossDirDataRealtimePO> insertCrossDirDataRealtimePOS = dealCrossDataDirRealTime(crossDirDataRealtimePOS, decimalFormat, producerHandler, startDate);
            Map<String, List<CrossDirDataRealtimePO>> dirDataMap = new HashMap<>();
            dirDataMap.put(Constant.REAL_TIME_DIR_DATA_KEY, insertCrossDirDataRealtimePOS);
            producerHandler.send2FirstKafka(Constant.REAL_TIME_DIR_DATA_TOPIC, dirDataMap);
            producerHandler.send2SecondKafka(Constant.REAL_TIME_DIR_DATA_TOPIC, dirDataMap);

            dealCrossDataRealTime(crossDataRealtimePOS, decimalFormat, producerHandler, startDate);
            dealCrossDataTurnRealTime(crossTurnDataRealtimePOS, decimalFormat, producerHandler, startDate);
        } catch (Exception e) {
            log.error("车道周期数据消费处理异常", e);
            throw new Exception(e);
        }

    }

    private static void dealCrossDataTurnRealTime(List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS, DecimalFormat decimalFormat, ProducerHandler producerHandler, Date startDate) {
        // 转向数据
        List<CrossTurnDataRealtimePO> insertCrossTurnDataRealtimePOS = new ArrayList<>();
        Map<String, List<CrossTurnDataRealtimePO>> crossTurnDataRealTimePOSMap = crossTurnDataRealtimePOS.stream().collect(Collectors.groupingBy(CrossTurnDataRealtimePO::getCrossId));
        for (Map.Entry<String, List<CrossTurnDataRealtimePO>> entry : crossTurnDataRealTimePOSMap.entrySet()) {
            String crossId = entry.getKey();
            List<CrossTurnDataRealtimePO> value = entry.getValue();

            Map<Integer, List<CrossTurnDataRealtimePO>> crossIdMap = value.stream().collect(Collectors.groupingBy(CrossTurnDataRealtimePO::getInDir));
            for (Map.Entry<Integer, List<CrossTurnDataRealtimePO>> item : crossIdMap.entrySet()) {
                Integer dir = item.getKey();

                List<CrossTurnDataRealtimePO> list = item.getValue();
                Map<String, List<CrossTurnDataRealtimePO>> turnMap = list.stream().collect(Collectors.groupingBy(CrossTurnDataRealtimePO::getTurnType));
                for (Map.Entry<String, List<CrossTurnDataRealtimePO>> turnEntry : turnMap.entrySet()) {
                    CrossTurnDataRealtimePO crossTurnDataRealTimePO = new CrossTurnDataRealtimePO();
                    List<CrossTurnDataRealtimePO> sameTurnList = turnEntry.getValue();
                    String turnType = turnEntry.getKey();
                    Integer inDir = null;
                    Integer outDir = null;
                    Integer flow = 0;
                    Double flowRate = 0.0;
                    Double speed = 0.0;
                    Double speedAverageTurn = 0.0;
                    int speedLaneCount = 0;
                    Double turnQueueLengthMax = 0.0;
                    Double stopTimes = 0.0;
                    Integer delayTime = 0;
                    Double sturation = 0.0;
                    Double noStopRate = 0.0;
                    Double oneStopRate = 0.0;
                    Double twoStopRate = 0.0;
                    Double threeStopRate = 0.0;
                    Double vehheadDist = 0.0;
                    Double vehheadTime = 0.0;
                    Double quality = 1.0;
                    Double effusionRate = 0.0;
                    Double greenLightEfficiency = 0.0;
                    for (CrossTurnDataRealtimePO crossTurnDataRealtimePO : sameTurnList) {
                        inDir = crossTurnDataRealtimePO.getInDir();
                        outDir = crossTurnDataRealtimePO.getOutDir();
                        turnType = crossTurnDataRealtimePO.getTurnType();
                        if (crossTurnDataRealtimePO.getFlow() > 0 && crossTurnDataRealtimePO.getSpeed() > 0.0) {
                            flow += crossTurnDataRealtimePO.getFlow();
                            speedAverageTurn += crossTurnDataRealtimePO.getSpeed();
                            ++speedLaneCount;
                        }
                        turnQueueLengthMax = turnQueueLengthMax >= crossTurnDataRealtimePO.getQueueLength() ? turnQueueLengthMax : crossTurnDataRealtimePO.getQueueLength();
                        stopTimes += crossTurnDataRealtimePO.getStopTimes();
                        delayTime += crossTurnDataRealtimePO.getDelayTime();
                        sturation += crossTurnDataRealtimePO.getSturation();
                        noStopRate += crossTurnDataRealtimePO.getNoStopRate();
                        oneStopRate += crossTurnDataRealtimePO.getOneStopRate();
                        twoStopRate += crossTurnDataRealtimePO.getTwoStopRate();
                        threeStopRate += crossTurnDataRealtimePO.getThreeStopRate();
                        vehheadDist += crossTurnDataRealtimePO.getVehheadDist();
                        vehheadTime += crossTurnDataRealtimePO.getVehheadTime();
                        effusionRate += crossTurnDataRealtimePO.getEffusionRate();
                        greenLightEfficiency = greenLightEfficiency >= crossTurnDataRealtimePO.getGreenLightEfficiency() ? greenLightEfficiency : crossTurnDataRealtimePO.getGreenLightEfficiency();
                    }
                    // 当路口出现异常情况，没有上报数据，上报数据都是0，计算车道默认1
                    if (speedLaneCount == 0) {
                        speedLaneCount = 1;
                    }
                    StringJoiner stringJoiner = new StringJoiner("");
                    String crossTurnId = stringJoiner.add(crossId).add(Constants.underline).add(String.valueOf(inDir))
                            .add(Constants.underline).add(turnType).toString();
                    crossTurnDataRealTimePO.setId(crossTurnId);
                    crossTurnDataRealTimePO.setCrossId(crossId);
                    crossTurnDataRealTimePO.setTurnType(turnType);
                    crossTurnDataRealTimePO.setInDir(inDir);
                    crossTurnDataRealTimePO.setOutDir(outDir);
                    crossTurnDataRealTimePO.setFlow(flow);
                    crossTurnDataRealTimePO.setQueueLength(Double.valueOf(decimalFormat.format(turnQueueLengthMax)));
                    crossTurnDataRealTimePO.setStopTimes(Double.valueOf(decimalFormat.format(stopTimes / speedLaneCount)));
                    crossTurnDataRealTimePO.setDelayTime(Integer.valueOf(decimalFormat.format(delayTime / speedLaneCount)));
                    crossTurnDataRealTimePO.setSturation(Double.valueOf(decimalFormat.format(sturation / speedLaneCount)));
                    crossTurnDataRealTimePO.setSpeed(Double.valueOf(decimalFormat.format(speedAverageTurn / speedLaneCount)));
                    crossTurnDataRealTimePO.setInSpeed(Double.valueOf(decimalFormat.format(speedAverageTurn / speedLaneCount))); //todo
                    crossTurnDataRealTimePO.setOutSpeed(Double.valueOf(decimalFormat.format(speedAverageTurn / speedLaneCount))); //todo
                    crossTurnDataRealTimePO.setNoStopRate(Math.min(Double.valueOf(decimalFormat.format(noStopRate / speedLaneCount)), 1.0));
                    crossTurnDataRealTimePO.setOneStopRate(Double.valueOf(decimalFormat.format(oneStopRate / speedLaneCount)));
                    crossTurnDataRealTimePO.setTwoStopRate(Double.valueOf(decimalFormat.format(twoStopRate / speedLaneCount)));
                    crossTurnDataRealTimePO.setThreeStopRate(Double.valueOf(decimalFormat.format(threeStopRate / speedLaneCount)));
                    crossTurnDataRealTimePO.setVehheadDist(Double.valueOf(decimalFormat.format(vehheadDist / speedLaneCount)));
                    crossTurnDataRealTimePO.setVehheadTime(Double.valueOf(decimalFormat.format(vehheadTime / speedLaneCount)));
                    crossTurnDataRealTimePO.setQuality(Double.valueOf(decimalFormat.format(quality)));
                    Integer batchTime = Integer.valueOf(String.valueOf(startDate.getTime()).substring(0, 10));
                    crossTurnDataRealTimePO.setBatchTime(batchTime);
                    crossTurnDataRealTimePO.setStartTime(startDate);
                    crossTurnDataRealTimePO.setGmtCreate(new Date());
                    crossTurnDataRealTimePO.setGmtModified(new Date());
                    crossTurnDataRealTimePO.setEffusionRate(Double.valueOf(decimalFormat.format(effusionRate / speedLaneCount)));
                    crossTurnDataRealTimePO.setGreenLightEfficiency(Double.valueOf(decimalFormat.format(greenLightEfficiency)));
                    // 默认速度
                    Double turnFreeFlowSpeed = WeekTurnFreeFlowSpeedCache.crossTurnFreeFlowSpeedMap.get(crossTurnId);
                    if (Objects.isNull(turnFreeFlowSpeed) || turnFreeFlowSpeed == 0.0) {
                        turnFreeFlowSpeed = 60.0;
                    }
                    if (speedAverageTurn == 0.0) {
                        speedAverageTurn = turnFreeFlowSpeed;
                    }
                    // 当前方向的进口延时指数 自由流速度/实际速度
                    Double turnDelayTimeIndex = turnFreeFlowSpeed / (speedAverageTurn / speedLaneCount);
                    int status = getDirStatus(turnDelayTimeIndex);
                    crossTurnDataRealTimePO.setStatus(status);
                    crossTurnDataRealTimePO.setTrafficIndex(Double.valueOf(decimalFormat.format(turnDelayTimeIndex)));
                    // 计算转向持续时间
                    if (status != 1) {
                        CrossTurnDataRealtimePO crossTurnDataRealTimePO4DB = getCrossTurnDataRealTimePO(crossId, turnType);
                        if (Objects.nonNull(crossTurnDataRealTimePO4DB)) {
                            Date startTime = crossTurnDataRealTimePO4DB.getStartTime();
                            crossTurnDataRealTimePO.setStartTime(startTime);
                            Date gmtCreate = crossTurnDataRealTimePO4DB.getGmtCreate();
                            crossTurnDataRealTimePO.setGmtCreate(gmtCreate);
                        }
                    }
                    insertCrossTurnDataRealtimePOS.add(crossTurnDataRealTimePO);
                }
            }
        }
        HashMap<String, List<CrossTurnDataRealtimePO>> turnDataMap = new HashMap<>();
        turnDataMap.put(Constant.REAL_TIME_TURN_DATA_KEY, insertCrossTurnDataRealtimePOS);
        producerHandler.send2FirstKafka(Constant.REAL_TIME_TURN_DATA_TOPIC, turnDataMap);
        producerHandler.send2SecondKafka(Constant.REAL_TIME_TURN_DATA_TOPIC, turnDataMap);
    }

    private List<CrossDirDataRealtimePO> dealCrossDataDirRealTime(List<CrossDirDataRealtimePO> crossDirDataRealtimePOS,
                                                                  DecimalFormat decimalFormat, ProducerHandler producerHandler, Date startDate) throws Exception {
        // 方向数据
        List<CrossDirDataRealtimePO> insertCrossDirDataRealtimePOS = new ArrayList<>();
        Map<String, List<CrossDirDataRealtimePO>> crossDirDataRealTimePOSMap = crossDirDataRealtimePOS.stream()
                .collect(Collectors.groupingBy(CrossDirDataRealtimePO::getCrossId));
        for (Map.Entry<String, List<CrossDirDataRealtimePO>> entry : crossDirDataRealTimePOSMap.entrySet()) {
            String crossId = entry.getKey();
            List<CrossDirDataRealtimePO> value = entry.getValue();
            Map<Integer, List<CrossDirDataRealtimePO>> crossIdMap = value.stream()
                    .collect(Collectors.groupingBy(CrossDirDataRealtimePO::getDirType));
            Double congestionIndex = 0.0;
            Double spilloverIndex = 0.0;
            Double saturationAverage = 0.0;
            Double saturationSum = 0.0;
            String unbalanceDirs = "";
            String congestionDirs = "";
            String spilloverDirs = "";
            List<Double> delayTimeList = new ArrayList<>();
            for (Map.Entry<Integer, List<CrossDirDataRealtimePO>> dirEntry : crossIdMap.entrySet()) {
                CrossDirDataRealtimePO crossDirDataRealTimePO = new CrossDirDataRealtimePO();
                Integer dir = dirEntry.getKey();
                List<CrossDirDataRealtimePO> dirList = dirEntry.getValue();
                // 每个方向计算交通流量累加，筛选最大停车长度
                Integer flow = 0;
                Double speedSum = 0.0;
                int speedLaneCount = 0; // 车道速度大于0的车道数量
                Double dirQueueLengthMax = 0.0;
                Double dirQueueLengthMin = 0.0;
                Double stopTimes = 0.0;
                Integer delayTime = 0;
                Integer capacity = 0;
                Double noStopRate = 0.0;
                Double oneStopRate = 0.0;
                Double twoStopRate = 0.0;
                Double threeStopRate = 0.0;
                Double effusionRate = 0.0;
                Double greenLightEfficiency = 0.0;
                for (CrossDirDataRealtimePO crossDirDataRealtimePOEntry : dirList) {
                    if (crossDirDataRealtimePOEntry.getSpeed() > 0 && crossDirDataRealtimePOEntry.getFlow() > 0) {
                        speedSum += crossDirDataRealtimePOEntry.getSpeed();
                        ++speedLaneCount;
                    }
                    flow += crossDirDataRealtimePOEntry.getFlow();
                    dirQueueLengthMax = dirQueueLengthMax >= crossDirDataRealtimePOEntry.getQueueLength() ? dirQueueLengthMax : crossDirDataRealtimePOEntry.getQueueLength();
                    dirQueueLengthMin = dirQueueLengthMin <= crossDirDataRealtimePOEntry.getQueueLength() ? dirQueueLengthMin : crossDirDataRealtimePOEntry.getQueueLength();
                    stopTimes += crossDirDataRealtimePOEntry.getStopTimes();
                    delayTime += crossDirDataRealtimePOEntry.getDelayTime();
                    noStopRate += crossDirDataRealtimePOEntry.getNoStopRate();
                    oneStopRate += crossDirDataRealtimePOEntry.getOneStopRate();
                    twoStopRate += crossDirDataRealtimePOEntry.getTwoStopRate();
                    threeStopRate += crossDirDataRealtimePOEntry.getThreeStopRate();
                    capacity += crossDirDataRealtimePOEntry.getCapacity();
                    effusionRate += crossDirDataRealtimePOEntry.getEffusionRate() == null ? 0.0 : crossDirDataRealtimePOEntry.getEffusionRate();
                    greenLightEfficiency = greenLightEfficiency >= crossDirDataRealtimePOEntry.getGreenLightEfficiency() ? greenLightEfficiency : crossDirDataRealTimePO.getGreenLightEfficiency();

                }
                Double dirSaturation = 0.0; // 通过方向七天流量最大值计算饱和度
                Map<String, Integer> crossWeekMaxFlowMap = CrossDirWeekMaxFlowCache.crossDirWeekMaxFlowMap;
                if (Objects.nonNull(crossWeekMaxFlowMap) && !crossWeekMaxFlowMap.isEmpty()) {
                    Integer dirMaxFlow = crossWeekMaxFlowMap.get(crossId.concat(Constants.underline).concat(String.valueOf(dir)));
                    if (Objects.isNull(dirMaxFlow) || dirMaxFlow == 0) {
                        dirMaxFlow = 100;
                    }
                    dirSaturation = (double) flow / dirMaxFlow;
                }
                crossDirDataRealTimePO.setSturation((Double.valueOf(decimalFormat.format(dirSaturation))));
                // 平均饱和度
                Double dirFlowAvg = CrossDirWeekAvgFlowCache.crossDirWeekAvgFlowMap.get(crossId.concat(Constants.underline).concat(String.valueOf(dir)));
                if (Objects.nonNull(dirFlowAvg)) {
                    saturationAverage = flow / dirFlowAvg;
                }
                Double dirLoadBalance = Math.pow((dirSaturation - saturationAverage) * (dirSaturation - saturationAverage), 2);
                // 所有方向饱和度平方后求和
                saturationSum += (dirSaturation - saturationAverage) * (dirSaturation - saturationAverage);
                StringJoiner stringJoiner = new StringJoiner("");
                stringJoiner.add(crossId).add(Constants.underline).add(String.valueOf(dir)).add(Constants.underline).add("1").add(Constants.underline).add("1");
                crossDirDataRealTimePO.setId(stringJoiner.toString());
                crossDirDataRealTimePO.setCrossId(crossId);
                crossDirDataRealTimePO.setDirType(dir);
                crossDirDataRealTimePO.setInOutType(1);
                double length = getDirRidLength(crossId, dir);
                crossDirDataRealTimePO.setLength(Double.valueOf(decimalFormat.format(length)));
                crossDirDataRealTimePO.setFlow(flow);
                if (speedLaneCount == 0) {
                    speedLaneCount = 1;
                }
                Double speedAverageDir = speedSum / speedLaneCount;
                crossDirDataRealTimePO.setSpeed(Double.valueOf(decimalFormat.format(speedAverageDir)));
                crossDirDataRealTimePO.setQueueLength(Double.valueOf(decimalFormat.format(dirQueueLengthMax)));
                crossDirDataRealTimePO.setStopTimes(Double.valueOf(decimalFormat.format(stopTimes / speedLaneCount)));
                crossDirDataRealTimePO.setDelayTime(Integer.valueOf(decimalFormat.format(delayTime / speedLaneCount)));
                crossDirDataRealTimePO.setCapacity(capacity);
                crossDirDataRealTimePO.setEffusionRate(Double.valueOf(decimalFormat.format(effusionRate / speedLaneCount)));
                crossDirDataRealTimePO.setNoStopRate(Math.min(Double.valueOf(decimalFormat.format(noStopRate / speedLaneCount)), 1.0));
                crossDirDataRealTimePO.setOneStopRate(Double.valueOf(decimalFormat.format(oneStopRate / speedLaneCount)));
                crossDirDataRealTimePO.setTwoStopRate(Double.valueOf(decimalFormat.format(twoStopRate / speedLaneCount)));
                crossDirDataRealTimePO.setThreeStopRate(Double.valueOf(decimalFormat.format(threeStopRate / speedLaneCount)));
                crossDirDataRealTimePO.setStartTime(startDate);
                Integer batchTime = Integer.valueOf(String.valueOf(startDate.getTime()).substring(0, 10));
                crossDirDataRealTimePO.setBatchTime(batchTime);
                crossDirDataRealTimePO.setGmtCreate(new Date());
                crossDirDataRealTimePO.setGmtModified(new Date());
                crossDirDataRealTimePO.setEffusionRate(Double.valueOf(decimalFormat.format(effusionRate / speedLaneCount)));
                crossDirDataRealTimePO.setGreenLightEfficiency(Double.valueOf(decimalFormat.format(greenLightEfficiency)));
                // 当前方向的进口延时指数
                Double dirDelayTimeIndex = 1.0;
                Integer status = 0;
                if (Constants.X_DIR_LIST.contains(dir)) { // 东西方向自由流计算延时指数
                    Map<String, Double> crossFreeFlowSpeedMap = WeekDirFreeFlowSpeedCache.crossDirFreeFlowSpeedMap;
                    if (!crossFreeFlowSpeedMap.isEmpty()) {
                        Double freeFlowSpeed = crossFreeFlowSpeedMap.get(crossId.concat(Constants.underline).concat(String.valueOf(dir)));
                        if (Objects.isNull(freeFlowSpeed) || freeFlowSpeed == 0.0) {
                            freeFlowSpeed = 60.0;
                        }
                        if (speedAverageDir == 0.0 || dirQueueLengthMax < 20.0) { // 夜间红灯速度小，排队长度小给自由流
                            speedAverageDir = freeFlowSpeed;
                        }
                        dirDelayTimeIndex = freeFlowSpeed / speedAverageDir;
                    }
                } else { // 南北通过饱和度
                    dirDelayTimeIndex = getDelayTimeIndex4Saturation(dirSaturation);
                }
                delayTimeList.add(dirDelayTimeIndex);

                Integer inputLaneNum = dirList.size();
                Double inputLaneIndex = (double) inputLaneNum / (double) value.size();
                // 拥堵指数
                Double dirCongextionIndex = inputLaneIndex * (dirDelayTimeIndex);
                congestionIndex += dirCongextionIndex;
                if (dirDelayTimeIndex > 1.55) { // 失衡，四个方向失衡就拥堵
                    unbalanceDirs = unbalanceDirs.concat(String.valueOf(dir).concat(","));
                    congestionDirs = congestionDirs.concat(String.valueOf(dir).concat(","));
                    status = 1;
                }
                // 溢出指数
                RidInfoEntity ridInfoEntity = CrossRidInfoCache.getStartRidInfoEntity(crossId, dir);
                Double crossRidLength = 0.0;
                Double dirSpilloverIndex = 0.0;
                if (Objects.nonNull(ridInfoEntity)) {
                    crossRidLength = ridInfoEntity.getLength();
                    dirSpilloverIndex = dirQueueLengthMax / crossRidLength;
                    if (dirSpilloverIndex >= 0.0 && dirSpilloverIndex <= crossRidLength / 3) {
                        dirSpilloverIndex = 1.0;
                    } else if (dirSpilloverIndex > crossRidLength / 3 && dirSpilloverIndex <= crossRidLength) {
                        dirSpilloverIndex = 3 * dirQueueLengthMax / crossRidLength;
                    }
                }
                spilloverIndex = spilloverIndex >= dirSpilloverIndex ? spilloverIndex : dirSpilloverIndex;
                Double trafficIndexDir = dirDelayTimeIndex * 0.4 + dirCongextionIndex * 0.4 + spilloverIndex * 0.2;

                if (trafficIndexDir > 1.0) {
                    crossDirDataRealTimePO.setTrafficIndex(Double.valueOf(decimalFormat.format(trafficIndexDir)));
                } else {
                    crossDirDataRealTimePO.setTrafficIndex(Double.valueOf(decimalFormat.format(1)));
                }
                crossDirDataRealTimePO.setStatus(status);
                crossDirDataRealTimePO.setDuration(5);
                // 修改持续时间
                if (status != 0) {
                    CrossDirDataRealtimePO crossDirDataRealTimePODB = getCrossDirDataRealTimePO(crossId, dir);
                    if (Objects.nonNull(crossDirDataRealTimePODB) && Objects.equals(status, crossDirDataRealTimePODB.getStatus())) {
                        Date startTime = crossDirDataRealTimePODB.getStartTime();
                        crossDirDataRealTimePO.setStartTime(startTime);
                        Date gmtCreate = crossDirDataRealTimePODB.getGmtCreate();
                        crossDirDataRealTimePO.setGmtCreate(gmtCreate);
                        long time = startDate.getTime() - startTime.getTime();
                        Integer duration = (int) time / 1000 / 60 + 5;
                        crossDirDataRealTimePO.setDuration(duration);
                    }
                }
                // 添加相位空放
                List<PhaseEmptyResult> phaseEmptyResultList = ConsumerHandler.phaseEmptyResultList;
                if (!CollectionUtils.isEmpty(phaseEmptyResultList) && status == 0) {
                    for (PhaseEmptyResult phaseEmptyResult : phaseEmptyResultList) {
                        if (Objects.equals(phaseEmptyResult.getCrossId(), crossId) &&
                                Objects.equals(phaseEmptyResult.getDirection(), String.valueOf(dir))) {
                            crossDirDataRealTimePO.setStatus(5);
                            crossDirDataRealTimePO.setStartTime(new Date(phaseEmptyResult.getStartTime()));
                        }
                    }
                }

                // 用路口事件数据替换部分字段
                replaceDirPartFields(crossDirDataRealTimePO, startDate);

                insertCrossDirDataRealtimePOS.add(crossDirDataRealTimePO);
            }
            CrossRealTimeCache.crossDelayTimeIndexMap.put(crossId, delayTimeList);
            Double delayTimeIndexMax = 0.0;
            Double delayTimeIndexMin = 0.0;
            if (!CollectionUtils.isEmpty(delayTimeList)) {
                delayTimeIndexMax = delayTimeList.stream().mapToDouble(Double::doubleValue).max().getAsDouble();
                delayTimeIndexMin = delayTimeList.stream().mapToDouble(Double::doubleValue).min().getAsDouble();
            }
            if (delayTimeIndexMin <= 0.0) {
                delayTimeIndexMin = 1.0;
            }
            // 负载均衡度
            Double loadBalance = Math.sqrt(saturationSum / crossIdMap.size());
            CrossDataIndexDTO crossDataIndexDTO = new CrossDataIndexDTO();
            crossDataIndexDTO.setCongestionIndex(Double.valueOf(decimalFormat.format(congestionIndex)));
            crossDataIndexDTO.setCongestionDirs(congestionDirs);
            crossDataIndexDTO.setUnbalanceIndex(Double.valueOf(decimalFormat.format(delayTimeIndexMax / delayTimeIndexMin)));
            crossDataIndexDTO.setUnbalanceDirs(unbalanceDirs);
            crossDataIndexDTO.setSpillOverIndex(Double.valueOf(decimalFormat.format(spilloverIndex)));
            crossDataIndexDTO.setSpillOverDirs(spilloverDirs);
            crossDataIndexDTO.setLoadBalance(Double.valueOf(decimalFormat.format(loadBalance)));
            CrossRealTimeCache.crossDataIndexMap.put(crossId, crossDataIndexDTO);
        }
        saveGreenWaveRealTime(insertCrossDirDataRealtimePOS);
        return insertCrossDirDataRealtimePOS;
    }

    /**
     * 获取路口方向的路段长度
     * @param crossId
     * @param dir
     * @return
     */
    private double getDirRidLength(String crossId, Integer dir) {
        RidInfoEntity endRidInfoEntity = CrossRidInfoCache.getEndRidInfoEntity(crossId, dir);
        if (Objects.nonNull(endRidInfoEntity)) {
            return endRidInfoEntity.getLength();
        }
        return 0.0;
    }

    private void replaceDirPartFields(CrossDirDataRealtimePO po, Date startDate) {
        String crossId = po.getCrossId();
        Integer dirType = po.getDirType();
        String redisKey = crossId + ":" + dirType;

        // 从 Redis 里读取当前时间窗口路口事件
        long startScore = startDate.getTime() + ConsumerHandler.TTL_IN_MILLIS; // score 是当前时间向后10分钟
        long endScore = startScore + 5 * 60 * 1000; // 时间窗口为5分钟
        Set<Object> zSetElements = redisUtils.getZsetRange(redisKey, startScore, endScore);
        if (ObjectUtil.isEmpty(zSetElements)) {
            return;
        }

        List<CrossEventDTO> crossEventDTOList = new ArrayList<>();
        ObjectMapper objectMapper = new ObjectMapper();
        for (Object element : zSetElements) {
            try {
                // 将 JSON 字符串转换成 CrossEventDTO 对象
                CrossEventDTO crossEventDTO = objectMapper.readValue(element.toString(), CrossEventDTO.class);
                crossEventDTOList.add(crossEventDTO);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if (ObjectUtil.isEmpty(crossEventDTOList)) {
            return;
        }

        // 每一种拥堵类型取最近的一条
        List<CrossEventDTO> congestEvents = crossEventDTOList.stream()
                .filter(crossEventDTO -> crossEventDTO.getMsgType() != null
                        && crossEventDTO.getMsgType().startsWith("50"))
                .collect(Collectors.groupingBy(CrossEventDTO::getMsgType))
                .entrySet().stream()
                .map(entry -> entry.getValue().stream()
                        .max(Comparator.comparingLong(CrossEventDTO::getDetectTime))
                        .orElse(null))
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        if (CollectionUtils.isEmpty(congestEvents)) {
            return;
        }
        // 找出最优事件
        CrossEventDTO crossEventDTO = congestEvents.stream()
                // 自定义比较器，按照 msgType 优先级排序
                .sorted(Comparator.comparingInt((CrossEventDTO e) -> {
                    switch (e.getMsgType()) {
                        case "504":
                            return 1;
                        case "503":
                            return 2;
                        case "502":
                            return 3;
                        case "501":
                            return 4;
                        default:
                            return 5;
                    }
                })).findFirst().get();
        // 替换交通状态，交通指数，开始时间，持续时长
        String msgType = crossEventDTO.getMsgType();
        Integer status = convertDirMsgType(msgType);
        po.setStatus(status);
        Double index = crossEventDTO.getIndex();
        po.setTrafficIndex(index);
        if (status != 1) { // 畅通事件无开始时间和持续时长
            Long startTime = crossEventDTO.getStartTime();
            po.setStartTime(new Date(startTime));
            Integer duration = crossEventDTO.getDuration();
            po.setDuration(duration);
        }
    }

    private Integer convertDirMsgType(String msgType) {
        if (msgType.equals(EventAbnormalEnum.NO_CONGEST.getType())) {
            return 1;
        } else if (msgType.equals(EventAbnormalEnum.LIGHT_CONGEST.getType())) {
            return 2;
        } else if (msgType.equals(EventAbnormalEnum.MODERATE_CONGEST.getType())) {
            return 3;
        } else if (msgType.equals(EventAbnormalEnum.HEAVY_CONGEST.getType())) {
            return 4;
        } else {
            return 5;
        }
    }

    private void saveGreenWaveRealTime(List<CrossDirDataRealtimePO> crossDirDataRealtimePOS) throws Exception {
        try {
            Map<Integer, List<GreenwaveCrossPO>> greenWaveInfoMap = CrossGreenWaveInfoCache.greenWaveInfoMap;
            if (!greenWaveInfoMap.isEmpty()) {
                List<GreenwaveRealtimePO> greenwaveRealtimePOS = new ArrayList<>();
                List<GreenwaveHistPO> greenwaveHistPOS = new ArrayList<>();
                for (Map.Entry<Integer, List<GreenwaveCrossPO>> entry : greenWaveInfoMap.entrySet()) {
                    List<GreenwaveCrossPO> greenWaveCrossIds = entry.getValue();
                    if (!CollectionUtils.isEmpty(greenWaveCrossIds)) {
                        int status = 1;
                        double trafficIndex = 1.0;
                        Double speedSum = 0.0;
                        Double stopTimes = 0.0;
                        double queueLength = 0.0;
                        double congRate = 0.0;
                        double congCount = 0.0;
                        Date startTime = null;
                        Integer delayTime = 0;
                        Double noparkPassRate = 0.0;
                        Integer greenWaveId = 0;
                        double speedAvg = 0.0;
                        double stopTimeAvg = 0.0;
                        double noparkPassRateAvg = 1.0;
                        Double greenLength = 1.0;
                        GreenwaveRealtimePO greenwaveRealtimePO = new GreenwaveRealtimePO();
                        for (GreenwaveCrossPO greenWaveCrossId : greenWaveCrossIds) {
                            String crossId = greenWaveCrossId.getCrossId();
                            Integer inDir = greenWaveCrossId.getInDir();
                            Integer sort = greenWaveCrossId.getSort();
                            greenWaveId = greenWaveCrossId.getGreenId();
                            if (Objects.equals(sort, 1)) { // 第一个路口不需要判断
                                continue;
                            }
                            GreenwaveInfoPO greenwaveInfoPO = getGreenWaveInfoPO(greenWaveId);
                            if (Objects.nonNull(greenwaveInfoPO)) {
                                greenLength = greenwaveInfoPO.getLength();
                            }
                            for (CrossDirDataRealtimePO dirDataRealtimePO : crossDirDataRealtimePOS) {
                                String crossIdDir = dirDataRealtimePO.getCrossId();
                                Integer dirType = dirDataRealtimePO.getDirType();
                                startTime = crossDirDataRealtimePOS.get(0).getStartTime();
                                if (StringUtils.equals(crossIdDir, crossId) && Objects.equals(inDir, dirType)) {
                                    status = status > dirDataRealtimePO.getStatus() ? status : dirDataRealtimePO.getStatus();
                                    trafficIndex = trafficIndex > dirDataRealtimePO.getTrafficIndex() ? trafficIndex : dirDataRealtimePO.getTrafficIndex();
                                    speedSum += dirDataRealtimePO.getSpeed();
                                    stopTimes += dirDataRealtimePO.getStopTimes();
                                    queueLength = queueLength > dirDataRealtimePO.getQueueLength() ? queueLength : dirDataRealtimePO.getQueueLength();
                                    delayTime += dirDataRealtimePO.getDelayTime();
                                    noparkPassRate += dirDataRealtimePO.getNoStopRate();
                                    if (dirDataRealtimePO.getStatus() >= 3) {
                                        ++congCount;
                                    }
                                }
                            }
                            //log.error("所有路口速度和：{}，路口方向列表：{}，绿波路口列表：{}", speedSum, crossDirDataRealtimePOS.size(), greenWaveCrossIds.size());
                            speedAvg = speedSum / (greenWaveCrossIds.size());
                            stopTimeAvg = stopTimes / (greenWaveCrossIds.size());
                            noparkPassRateAvg = noparkPassRate / (greenWaveCrossIds.size());
                            congRate = congCount / greenWaveCrossIds.size();
                        }
                        greenwaveRealtimePO.setGreenId(greenWaveId);
                        greenwaveRealtimePO.setStartTime(startTime);
                        greenwaveRealtimePO.setStatus(status);
                        greenwaveRealtimePO.setType(1);
                        greenwaveRealtimePO.setTrafficIndex(trafficIndex);
                        greenwaveRealtimePO.setSpeed(speedAvg);
                        greenwaveRealtimePO.setTrvalTime((int) (greenLength / speedAvg * 3.6));
                        greenwaveRealtimePO.setStopTimes(stopTimeAvg);
                        greenwaveRealtimePO.setQueueLength(queueLength);
                        greenwaveRealtimePO.setCongRate(congRate);
                        greenwaveRealtimePO.setDelayTime(delayTime);
                        greenwaveRealtimePO.setNoparkPassRate(noparkPassRateAvg);
                        greenwaveRealtimePO.setCordQueueRatio(queueLength / greenLength); // 空间占比
                        greenwaveRealtimePO.setCordReliability(1.00); // 可靠性
                        greenwaveRealtimePO.setUncoordinatePhaseQueue(10.00); // 非协调二次排队
                        greenwaveRealtimePO.setGmtCreate(new Date());
                        greenwaveRealtimePO.setGmtModified(new Date());
                        greenwaveRealtimePOS.add(greenwaveRealtimePO);
                        replaceGreenwavePartFields(greenwaveRealtimePO);
                        GreenwaveHistPO greenwaveHistPO = new GreenwaveHistPO();
                        BeanUtils.copyProperties(greenwaveRealtimePO, greenwaveHistPO);
                        greenwaveHistPOS.add(greenwaveHistPO);
                        greenwaveRealtimeMapper.deleteOne(greenWaveId);
                    }
                }
                if (!CollectionUtils.isEmpty(greenwaveHistPOS)) {
                    greenwaveHistMapper.insertBatch(greenwaveHistPOS);
                    greenwaveRealtimeMapper.insertBatch(greenwaveRealtimePOS);
                }
            }
        } catch (Exception e) {
            log.error("插入绿波实时历史表失败：", e);
            throw new Exception(e);
        }
    }

    private void replaceGreenwavePartFields(GreenwaveRealtimePO po) {
        String redisKey = "line_congestion_sink";
        Date startTime = po.getStartTime();
        // 从 Redis 里读取当前时间窗口路口事件
        long startScore = startTime.getTime() + ConsumerHandler.TTL_IN_MILLIS; // score 是当前时间之后10分钟
        long endScore = startScore + 5 * 60 * 1000; // 时间窗口为5分钟
        Set<Object> zSetElements = redisUtils.getZsetRange(redisKey, startScore, endScore);
        if (ObjectUtil.isEmpty(zSetElements)) {
            return;
        }

        List<LineCongestion> dtoList = new ArrayList<>();
        ObjectMapper objectMapper = new ObjectMapper();
        for (Object element : zSetElements) {
            try {
                // 将 JSON 字符串转换成 LineCongestion 对象
                LineCongestion dto = JSONObject.parseObject(element.toString(), LineCongestion.class);//objectMapper.readValue(element.toString(), LineCongestion.class);
                dtoList.add(dto);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if (ObjectUtil.isEmpty(dtoList)) {
            return;
        }

        // 取最新的一条
        Optional<LineCongestion> latest = dtoList.stream()
                .max(Comparator.comparingLong(LineCongestion::getDetectTime));
        LineCongestion lineCongestion = latest.orElse(null);
        if (ObjectUtil.isEmpty(lineCongestion)) {
            return;
        }

        // 替换指数
        po.setTrafficIndex(lineCongestion.getIndex());
        // 替换状态
        String congestionCode = lineCongestion.getCongestionCode();
        if (EventAbnormalEnum.NO_CONGEST.getType().equals(congestionCode)) {
            po.setStatus(1);
        } else {
            po.setStatus(3);
            // 如果 lineCongestion 是拥堵，替换 po 开始时间，结束时间
            Long congestionStartTime = lineCongestion.getStartTime();
            po.setStartTime(new Date(congestionStartTime));
            Integer duration = lineCongestion.getDuration();
            long congestionEndMilli = congestionStartTime + duration * 60 * 1000;
            po.setGmtModified(new Date(congestionEndMilli));
        }
    }

    /**
     * 获取绿波信息
     *
     * @param greenWaveId
     * @return
     */
    private static GreenwaveInfoPO getGreenWaveInfoPO(Integer greenWaveId) {
        List<GreenwaveInfoPO> greenwaveInfoPOS = greenwaveInfoMapper.selectAll();
        if (!CollectionUtils.isEmpty(greenwaveInfoPOS)) {
            for (GreenwaveInfoPO greenwaveInfoPO : greenwaveInfoPOS) {
                if (Objects.equals(greenWaveId, greenwaveInfoPO.getId())) {
                    return greenwaveInfoPO;
                }
            }
        }
        return null;
    }

    /**
     * 通过饱和度计算的延时指数计算路口方向状态
     *
     * @param dirDelayTimeIndex
     * @return
     */
    private static Integer getDirStatus(Double dirDelayTimeIndex) {
        Integer status = 1;
        if (dirDelayTimeIndex <= 1.25) {
            status = 1;
        }
        if (dirDelayTimeIndex > 1.25 && dirDelayTimeIndex <= 1.55) {
            status = 2;
        }
        if (dirDelayTimeIndex > 1.55 && dirDelayTimeIndex <= 2.0) {
            status = 3;
        }
        if (dirDelayTimeIndex > 2 && dirDelayTimeIndex <= 2.8) {
            status = 4;
        }
        return status;
    }

    private static Double getDelayTimeIndex4Saturation(Double dirSaturation) {
        DecimalFormat decimalFormat = new DecimalFormat("#.#");
        dirSaturation = Double.valueOf(decimalFormat.format(dirSaturation));
        Double index = 1.0;
        if (dirSaturation < 0.6) {
            index = 1.0;
        }
        if (dirSaturation >= 0.6 && dirSaturation < 0.7) {
            index = 1.55;
        }
        if (dirSaturation >= 0.7 && dirSaturation < 0.8) {
            index = 2.0; // 失衡
        }
        if (dirSaturation >= 0.8 && dirSaturation <= 0.9) {
            index = 2.28;
        }
        if (dirSaturation > 0.9) {
            index = 2.8;
        }
        return index;
    }

    /**
     * 通过拥堵指数判断路口拥堵状态
     *
     * @param status
     * @param dirCongextionIndex
     * @return
     */
    private static Integer getCongextionStatus(Double dirCongextionIndex) {
        Integer status = 5; // 未知
        if (dirCongextionIndex <= 1) {
            status = 1; // 畅通
        }
        if (dirCongextionIndex > 1 && dirCongextionIndex <= 3) {
            status = 2; // 缓行
        }
        if (dirCongextionIndex > 2 && dirCongextionIndex < 3) {
            status = 3; // 拥堵
        }
        if (dirCongextionIndex >= 3) {
            status = 4;
        }
        return status;
    }

    private void dealCrossDataRealTime(List<CrossDataRealtimePO> crossDataRealtimePOS, DecimalFormat decimalFormat, ProducerHandler producerHandler, Date startDate) {
        // 将车道转换的数据通过路口，方向，转向维度计算，存储
        List<CrossDataRealtimePO> insertCrossDataRealtimePOS = new ArrayList<>();
        Map<String, List<CrossDataRealtimePO>> crossDataRealTimePOSMap = crossDataRealtimePOS.stream().collect(Collectors.groupingBy(CrossDataRealtimePO::getCrossId));
        for (Map.Entry<String, List<CrossDataRealtimePO>> entry : crossDataRealTimePOSMap.entrySet()) {
            String crossId = entry.getKey();
            List<CrossDataRealtimePO> crossList = entry.getValue();
            CrossDataRealtimePO insertCrossDataRealtimePO = new CrossDataRealtimePO();
            Integer status = 0;
            Integer flow = 0;
            Double speed = 0.0;
            Double queueLengthMax = 0.0;
            Double queueLengthMin = 0.0;
            Double stopTimes = 0.0;
            Integer delayTime = 0;
            Double sturation = 0.0;
            Double noStopRate = 0.0;
            Double oneStopRate = 0.0;
            Double twoStopRate = 0.0;
            Double threeStopRate = 0.0;
            Double effusionRate = 0.0;
            Double greenLightEfficiency = 0.0;
            int speedLaneCount = 0;
            for (CrossDataRealtimePO crossDataRealTimePO : crossList) {
                // 去除车道流量为0，并且速度为0
                if (crossDataRealTimePO.getFlow() > 0 && crossDataRealTimePO.getSpeed() > 0.0) {
                    flow += crossDataRealTimePO.getFlow();
                    speed += crossDataRealTimePO.getSpeed();
                    ++speedLaneCount;
                }
                queueLengthMax = queueLengthMax >= crossDataRealTimePO.getQueueLength() ? queueLengthMax : crossDataRealTimePO.getQueueLength();
                queueLengthMin = queueLengthMin <= crossDataRealTimePO.getQueueLength() ? queueLengthMin : crossDataRealTimePO.getQueueLength();
                stopTimes += crossDataRealTimePO.getStopTimes();
                delayTime += crossDataRealTimePO.getDelayTime();
                sturation += crossDataRealTimePO.getSturation();
                effusionRate += crossDataRealTimePO.getEffusionRate() == null ? 0.0 : crossDataRealTimePO.getEffusionRate();
                effusionRate += crossDataRealTimePO.getEffusionRate();
                greenLightEfficiency = greenLightEfficiency >= crossDataRealTimePO.getGreenLightEfficiency() ? greenLightEfficiency : crossDataRealTimePO.getGreenLightEfficiency();
                noStopRate += crossDataRealTimePO.getNoStopRate();
                oneStopRate += crossDataRealTimePO.getOneStopRate();
                twoStopRate += crossDataRealTimePO.getTwoStopRate();
                threeStopRate += crossDataRealTimePO.getThreeStopRate();
            }
            // 当路口出现异常情况，没有上报数据，上报数据都是0，计算车道默认1
            if (speedLaneCount <= 0) {
                speedLaneCount = 1;
            }
            insertCrossDataRealtimePO.setCrossId(crossId);
            insertCrossDataRealtimePO.setFlow(flow);
            insertCrossDataRealtimePO.setFlowRate(Double.valueOf(decimalFormat.format(flow * 12)));
            insertCrossDataRealtimePO.setSpeed(Double.valueOf(decimalFormat.format(speed / speedLaneCount)));
            insertCrossDataRealtimePO.setQueueLength(Double.valueOf(decimalFormat.format(queueLengthMax)));
            insertCrossDataRealtimePO.setStopTimes(Double.valueOf(decimalFormat.format(stopTimes / speedLaneCount)));
            insertCrossDataRealtimePO.setDelayTime(Integer.valueOf(decimalFormat.format(delayTime / speedLaneCount)));
            insertCrossDataRealtimePO.setSturation(Double.valueOf(decimalFormat.format(sturation / speedLaneCount)));
            insertCrossDataRealtimePO.setEffusionRate(Double.valueOf(decimalFormat.format(effusionRate / crossList.size())));
            insertCrossDataRealtimePO.setGreenLightEfficiency(Double.valueOf(decimalFormat.format(greenLightEfficiency)));
            insertCrossDataRealtimePO.setNoStopRate(Math.min(Double.valueOf(decimalFormat.format(noStopRate / speedLaneCount)), 1.0));
            insertCrossDataRealtimePO.setOneStopRate(Double.valueOf(decimalFormat.format(oneStopRate / speedLaneCount)));
            insertCrossDataRealtimePO.setTwoStopRate(Double.valueOf(decimalFormat.format(twoStopRate / speedLaneCount)));
            insertCrossDataRealtimePO.setThreeStopRate(Double.valueOf(decimalFormat.format(threeStopRate / speedLaneCount)));
            CrossDataIndexDTO crossDataIndexDTO = CrossRealTimeCache.crossDataIndexMap.get(crossId);
            Double congestionIndex = crossDataIndexDTO.getCongestionIndex() < 1.0 ? 1.0 : crossDataIndexDTO.getCongestionIndex();
            insertCrossDataRealtimePO.setCongestionIndex(congestionIndex > 10 ? 10 : congestionIndex);
            insertCrossDataRealtimePO.setIsCongestion(0);
            insertCrossDataRealtimePO.setCongestionDirs(crossDataIndexDTO.getCongestionDirs() == null ? "," : crossDataIndexDTO.getCongestionDirs());
            Double unbalanceIndex = crossDataIndexDTO.getUnbalanceIndex() < 1.0 ? 1.0 : crossDataIndexDTO.getUnbalanceIndex();
            insertCrossDataRealtimePO.setUnbalanceIndex(unbalanceIndex > 10 ? 10 : unbalanceIndex);
            insertCrossDataRealtimePO.setIsUnbalance(0);
            insertCrossDataRealtimePO.setUnbalanceDirs(crossDataIndexDTO.getUnbalanceDirs() == null ? "," : crossDataIndexDTO.getUnbalanceDirs());
            Double spillOverIndex = crossDataIndexDTO.getSpillOverIndex() < 1.0 ? 1.0 : crossDataIndexDTO.getSpillOverIndex();
            insertCrossDataRealtimePO.setSpilloverIndex(spillOverIndex > 10 ? 10 : spillOverIndex);
            insertCrossDataRealtimePO.setIsSpillover(spillOverIndex >= 2.4 ? 1 : 0);
            insertCrossDataRealtimePO.setSpilloverDirs(crossDataIndexDTO.getSpillOverDirs() == null ? "," : crossDataIndexDTO.getSpillOverDirs());
            // 延时指数列表
            List<Double> delayTimeIndexList = CrossRealTimeCache.crossDelayTimeIndexMap.get(crossId);
            // 失衡指数列表
            List<Double> unbalanceList = delayTimeIndexList.stream().filter(d -> d <= 1.55).collect(Collectors.toList());
            if (unbalanceList.size() == delayTimeIndexList.size()) {
                status = 0; // 所有进口小于1.55，正常；
            } else {
                status = 1;// 有部分进口小于1.55 失衡
                insertCrossDataRealtimePO.setIsUnbalance(1);
            }
            // 拥堵指数列表
            List<Double> congestionList = delayTimeIndexList.stream().filter(d -> d > 1.55).collect(Collectors.toList());
            if (congestionList.size() == delayTimeIndexList.size()) {
                status = 2; // 所有路口失衡，拥堵
                insertCrossDataRealtimePO.setIsCongestion(1);
            }
            insertCrossDataRealtimePO.setStatus(status);
            if (status == 0) {
                insertCrossDataRealtimePO.setType(1);
            } else {
                insertCrossDataRealtimePO.setType(2);
            }
            Double loadBalance = crossDataIndexDTO.getLoadBalance();
            insertCrossDataRealtimePO.setLoadBalance(loadBalance);
            insertCrossDataRealtimePO.setGmtCreate(new Date());
            insertCrossDataRealtimePO.setGmtModified(new Date());
            insertCrossDataRealtimePO.setStartTime(startDate);
            insertCrossDataRealtimePO.setEndTime(DateUtil.offsetMinute(startDate, 5));
            Integer batchTime = Integer.valueOf(String.valueOf(startDate.getTime()).substring(0, 10));
            insertCrossDataRealtimePO.setBatchTime(batchTime);
            // 获取未清空车道数量
            Integer notClearCarNums = getNotClearCarNums(crossId);
            Double clearRate = CrossRealTimeCache.crossPhaseIdClearGreenMap.get(crossId);
            if (Objects.isNull(clearRate)) {
                clearRate = 0.0;
            }
            if (notClearCarNums == 0) {
                clearRate = 1.0;
            }
            Integer allRed = periodHaveAllRedMap.get(crossId);
            if (!Objects.equals(allRed, 1)) { // 无全场红
                clearRate = -1.0;
            }
            insertCrossDataRealtimePO.setClearRate(clearRate);
            insertCrossDataRealtimePO.setNotClearCarNums(notClearCarNums);
            Double trafficIndex = unbalanceIndex * 0.4 + congestionIndex * 0.4 + spillOverIndex * 0.2;
            insertCrossDataRealtimePO.setTrafficIndex(Double.valueOf(decimalFormat.format(trafficIndex)));
            insertCrossDataRealtimePO.setDuration(5);
            if (status != 0) {
                CrossDataRealtimePO crossDataRealTimePO = getCrossDataRealTimePO(crossId);
                if (Objects.nonNull(crossDataRealTimePO) && Objects.equals(status, crossDataRealTimePO.getStatus())) {
                    Date startTime = crossDataRealTimePO.getStartTime();
                    insertCrossDataRealtimePO.setStartTime(startTime);
                    Date gmtCreate = crossDataRealTimePO.getGmtCreate();
                    insertCrossDataRealtimePO.setGmtCreate(gmtCreate);
                    long time = startDate.getTime() - startTime.getTime();
                    Integer duration = (int) time / 1000 / 60 + 5;
                    insertCrossDataRealtimePO.setDuration(duration);
                }
            }
            periodHaveAllRedMap.remove(crossId);

            // 添加相位空放
            if (!CollectionUtils.isEmpty(ConsumerHandler.phaseEmptyResultList) && insertCrossDataRealtimePO.getStatus() == 1) {
                for (PhaseEmptyResult phaseEmptyResult : ConsumerHandler.phaseEmptyResultList) {
                    if (phaseEmptyResult.getCrossId().equals(crossId)) {
                        insertCrossDataRealtimePO.setStatus(5);
                        insertCrossDataRealtimePO.setStartTime(new Date(phaseEmptyResult.getStartTime()));
                    }
                }
            }

            insertCrossDataRealtimePOS.add(insertCrossDataRealtimePO);

            replaceCrossPartFields(insertCrossDataRealtimePO, startDate);

        }
        Map<String, List<CrossDataRealtimePO>> crossDataMap = new HashMap<>();
        crossDataMap.put(Constant.REAL_TIME_CROSS_DATA_KEY, insertCrossDataRealtimePOS);
        producerHandler.send2FirstKafka(Constant.REAL_TIME_CROSS_DATA_TOPIC, crossDataMap);
        producerHandler.send2SecondKafka(Constant.REAL_TIME_CROSS_DATA_TOPIC, crossDataMap);
    }

    private void replaceCrossPartFields(CrossDataRealtimePO po, Date startDate) {
        String crossId = po.getCrossId();
        String redisKey = crossId;
        String foo;

        // 从 Redis 里读取当前时间窗口路口事件
        long startScore = startDate.getTime() + ConsumerHandler.TTL_IN_MILLIS; // score 是当前时间向后10分钟
        long endScore = startScore + 5 * 60 * 1000; // 时间窗口为5分钟
        Set<Object> zSetElements = redisUtils.getZsetRange(redisKey, startScore, endScore);
        if (ObjectUtil.isEmpty(zSetElements)) {
            return;
        }

        List<CrossEventDTO> crossEventDTOList = new ArrayList<>();
        ObjectMapper objectMapper = new ObjectMapper();
        for (Object element : zSetElements) {
            try {
                // 将 JSON 字符串转换成 CrossEventDTO 对象
                CrossEventDTO crossEventDTO = objectMapper.readValue(element.toString(), CrossEventDTO.class);
                crossEventDTOList.add(crossEventDTO);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if (ObjectUtil.isEmpty(crossEventDTOList)) {
            return;
        }

        // 每一种事件类型取最近的一条
        List<CrossEventDTO> crossEvents = crossEventDTOList.stream()
                .filter(crossEventDTO -> crossEventDTO.getMsgType() != null)
                .collect(Collectors.groupingBy(CrossEventDTO::getMsgType))
                .entrySet().stream()
                .map(entry -> entry.getValue().stream()
                        .max(Comparator.comparingLong(CrossEventDTO::getDetectTime))
                        .orElse(null))
                .filter(Objects::nonNull)
                .sorted(Comparator.comparing(CrossEventDTO::getMsgType)) // 根据事件严重程度排序
                .collect(Collectors.toList());

        if (ObjectUtil.isEmpty(crossEvents)) {
            return;
        }

        // 替换路口状态，交通指数，开始持续结束时间，是否失衡拥堵溢出，失衡拥堵溢出指数，失衡拥堵溢出方向
        for (CrossEventDTO crossEvent : crossEvents) {
            String msgType = crossEvent.getMsgType();
            Double index = crossEvent.getIndex();
            Integer status = convertCrossMsgType(msgType);
            Long startTime = crossEvent.getStartTime();
            Long endTime = crossEvent.getEndTime();

            if (msgType.equals(EventAbnormalEnum.NO_CONGEST.getType())) { // 畅通
                po.setStatus(status);

                po.setTrafficIndex(index);

                po.setIsUnbalance(0);
                po.setIsCongestion(0);
                po.setIsSpillover(0);

                po.setCongestionIndex(index);

                po.setUnbalanceDirs("");
                po.setCongestionDirs("");
                po.setSpilloverDirs("");
            }

            if (msgType.equals(EventAbnormalEnum.MODERATE_CONGEST.getType())
                    || msgType.equals(EventAbnormalEnum.HEAVY_CONGEST.getType())) { // 拥堵
                po.setStatus(status);

                po.setTrafficIndex(index);

                Date date = new Date(startTime);
                po.setStartTime(date);
                Integer duration = crossEvent.getDuration();
                po.setDuration(duration);
                Date date1 = new Date(endTime);
                po.setEndTime(date1);

                po.setIsCongestion(1);

                po.setCongestionIndex(index);

                po.setUnbalanceDirs("");
                po.setCongestionDirs(buildEventDirs(crossEvent));
                po.setSpilloverDirs("");
            }

            if (msgType.equals(EventAbnormalEnum.CROSS_UNBALANCE.getType())) { // 失衡
                // 检查是否有拥堵
                boolean hasCongest = checkHasCongest(crossEvents);
                if (!hasCongest) {
                    // 如果之前没有拥堵，则替换为失衡状态
                    po.setStatus(status);

                    Date date = new Date(startTime);
                    po.setStartTime(date);
                    Integer duration = crossEvent.getDuration();
                    po.setDuration(duration);
                    Date date1 = new Date(endTime);
                    po.setEndTime(date1);
                }
                po.setIsUnbalance(1);
                po.setUnbalanceIndex(index);

                Integer unbalanceDir = crossEvent.getUnbalanceDir();
                po.setUnbalanceDirs(unbalanceDir.toString());
            }

            if (msgType.equals(EventAbnormalEnum.CROSS_OVERFLOW.getType())) { // 溢出
                po.setStatus(status);

                Date date = new Date(startTime);
                po.setStartTime(date);
                Integer duration = crossEvent.getDuration();
                po.setDuration(duration);
                Date date1 = new Date(endTime);
                po.setEndTime(date1);

                po.setIsSpillover(1);
                po.setSpilloverIndex(index);

                po.setSpilloverDirs(buildEventDirs(crossEvent));
            }

            if (msgType.equals(EventAbnormalEnum.CROSS_DEADLOCK.getType())) { // 死锁
                po.setStatus(status);

                Date date = new Date(startTime);
                po.setStartTime(date);
                Integer duration = crossEvent.getDuration();
                po.setDuration(duration);
                Date date1 = new Date(endTime);
                po.setEndTime(date1);

                po.setIsSpillover(1);
                po.setSpilloverIndex(index);

                po.setSpilloverDirs(buildEventDirs(crossEvent));
            }
        }

    }

    private boolean checkHasCongest(List<CrossEventDTO> crossEvents) {
        boolean hasCongest = false;
        for (CrossEventDTO crossEvent : crossEvents) {
            String msgType = crossEvent.getMsgType();
            if (msgType.equals(EventAbnormalEnum.MODERATE_CONGEST.getType())
                    || msgType.equals(EventAbnormalEnum.HEAVY_CONGEST.getType())) {
                hasCongest = true;
            }
        }
        return hasCongest;
    }

    private String buildEventDirs(CrossEventDTO crossEvent) {
        List<CrossEventDTO> details = crossEvent.getDetails();
        String dirString = details.stream()
                .map(crossEventDTO -> crossEventDTO.getDir())
                .filter(Objects::nonNull) // 过滤掉任何可能的 null 值
                .map(String::valueOf) // 将 Integer 转换为 String
                .collect(Collectors.joining(",")); // 以逗号连接
        return dirString;
    }

    private Integer convertCrossMsgType(String msgType) {
        if (msgType.equals(EventAbnormalEnum.NO_CONGEST.getType())) {
            return 0;
        } else if (msgType.startsWith("50")) {
            return 2;
        } else if (msgType.equals(EventAbnormalEnum.CROSS_UNBALANCE.getType())) {
            return 1;
        } else if (msgType.equals(EventAbnormalEnum.CROSS_OVERFLOW.getType())) {
            return 3;
        } else if (msgType.equals(EventAbnormalEnum.CROSS_DEADLOCK.getType())) {
            return 4;
        } else {
            return 5;
        }
    }

    /**
     * 查询路口实时表是否状态一致，一致不更新开始时间
     *
     * @param crossId
     * @return CrossDataRealtimePO
     */
    private static CrossDataRealtimePO getCrossDataRealTimePO(String crossId) {
        LambdaQueryWrapper<CrossDataRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CrossDataRealtimePO::getCrossId, crossId);
        List<CrossDataRealtimePO> crossDataRealtimePOS = crossDataRealTimeMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(crossDataRealtimePOS)) {
            return crossDataRealtimePOS.get(0);
        }
        return null;
    }

    private static CrossDirDataRealtimePO getCrossDirDataRealTimePO(String crossId, Integer dir) {
        LambdaQueryWrapper<CrossDirDataRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CrossDirDataRealtimePO::getCrossId, crossId);
        queryWrapper.eq(CrossDirDataRealtimePO::getDirType, dir);
        List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = crossDirDataRealTimeMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(crossDirDataRealtimePOS)) {
            return crossDirDataRealtimePOS.get(0);
        }
        return null;
    }

    private static CrossTurnDataRealtimePO getCrossTurnDataRealTimePO(String crossId, String turnType) {
        LambdaQueryWrapper<CrossTurnDataRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CrossTurnDataRealtimePO::getCrossId, crossId);
        queryWrapper.eq(CrossTurnDataRealtimePO::getTurnType, turnType);
        List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = crossTurnDataRealTimeMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(crossTurnDataRealtimePOS)) {
            return crossTurnDataRealtimePOS.get(0);
        }
        return null;
    }


    /**
     * @param crossId
     * @return notClearCarNums
     * @description 获取缓存中路口的绿灯未清空车辆数量
     */
    private static Integer getNotClearCarNums(String crossId) {
        Integer notClearCarNums = 0;
        if (!CrossRealTimeCache.crossPhaseCarNumsMap.isEmpty()) {
            for (Map.Entry<String, Integer> item : CrossRealTimeCache.crossPhaseCarNumsMap.entrySet()) {
                String crossIdPhase = item.getKey();
                if (crossIdPhase.contains(crossId)) {
                    notClearCarNums += item.getValue();
                }
            }
        }
        return notClearCarNums;
    }
}
