package net.wanji.datacenter.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.po.LaneInfoPO;
import net.wanji.datacenter.cache.CrossLaneInfoCache;
import net.wanji.datacenter.common.Constants;
import net.wanji.datacenter.pojo.dto.CrossLaneSnapshotDataDTO;
import net.wanji.datacenter.pojo.dto.LaneSnapshotDataDTO;
import net.wanji.datacenter.service.DataProcessService;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.indices.CreateIndexRequest;
import org.elasticsearch.client.indices.CreateIndexResponse;
import org.elasticsearch.client.indices.GetIndexRequest;
import org.elasticsearch.common.xcontent.XContentType;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/10/25 17:27
 */
@Slf4j
@Service(value = "laneSnapshotDataProcessService")
public class LaneSnapshotDataProcessService implements DataProcessService {
    @Resource
    RestHighLevelClient client;

    @Override
    public void send(String topic, Object convertData) {

    }

    @Override
    public List<CrossLaneSnapshotDataDTO> convert(String originalData) throws Exception {
        List<CrossLaneSnapshotDataDTO> result = new ArrayList<>();
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            LaneSnapshotDataDTO laneSnapshotDataDTO = mapper.readValue(originalData, LaneSnapshotDataDTO.class);
            if (Objects.nonNull(laneSnapshotDataDTO)) {
                String timeStamp = laneSnapshotDataDTO.getTimeStamp();
                Date date = new SimpleDateFormat(DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()).parse(timeStamp);
                Integer batchTime = Integer.valueOf(String.valueOf(date.getTime()).substring(0, 10));
                List<LaneSnapshotDataDTO.EventList> eventList = laneSnapshotDataDTO.getEventList();
                if (!CollectionUtils.isEmpty(eventList)) {
                    for (LaneSnapshotDataDTO.EventList list : eventList) {
                        CrossLaneSnapshotDataDTO crossLaneSnapshotDataDTO = new CrossLaneSnapshotDataDTO();
                        String laneId = list.getLaneId();
                        LaneInfoPO laneInfoPO = CrossLaneInfoCache.laneInfoMap.get(laneId);
                        if (Objects.nonNull(laneInfoPO)) {
                            crossLaneSnapshotDataDTO.setId(laneInfoPO.getCrossId().concat(timeStamp));
                            crossLaneSnapshotDataDTO.setCrossId(laneInfoPO.getCrossId());
                            crossLaneSnapshotDataDTO.setDir(laneInfoPO.getDir());
                            crossLaneSnapshotDataDTO.setTurn(laneInfoPO.getTurn());
                            crossLaneSnapshotDataDTO.setTimeStamp(batchTime);
                            crossLaneSnapshotDataDTO.setLaneId(laneInfoPO.getId());
                            crossLaneSnapshotDataDTO.setStaticQueueLength(list.getStaticQueueLength());
                            crossLaneSnapshotDataDTO.setTeamHeadDistance(list.getTeamHeadDistance());
                            crossLaneSnapshotDataDTO.setTeamTailDistance(list.getTeamTailDistance());
                            crossLaneSnapshotDataDTO.setDynamicQueueLength(list.getDynamicQueueLength());
                            crossLaneSnapshotDataDTO.setMeanSpaceHeadway(list.getMeanSpaceHeadway());
                            crossLaneSnapshotDataDTO.setStdSpaceHeadway(list.getStdSpaceHeadway());
                            crossLaneSnapshotDataDTO.setQueueNums(list.getQueueNums());
                            crossLaneSnapshotDataDTO.setCarNums(list.getCarNums());
                            crossLaneSnapshotDataDTO.setVehicleLengthRatio(list.getVehicleLengthRatio());
                            crossLaneSnapshotDataDTO.setVehicleNumsRatio(list.getVehicleNumsRatio());
                            crossLaneSnapshotDataDTO.setMeanV(list.getMeanV());
                            crossLaneSnapshotDataDTO.setHeadCarPosition(list.getHeadCarPosition());
                            crossLaneSnapshotDataDTO.setStartV(list.getStartV());
                            crossLaneSnapshotDataDTO.setTailCarPosition(list.getTailCarPosition());
                            crossLaneSnapshotDataDTO.setEndV(list.getEndV());
                            crossLaneSnapshotDataDTO.setOverflowNums(list.getOverflowNums());
                            crossLaneSnapshotDataDTO.setOverflow(list.isOverflow());
                            result.add(crossLaneSnapshotDataDTO);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("车道快照数据转换异常", e);
            throw new Exception(e);
        }
        return result;
    }

    @Override
    public void save(Object convertData) throws Exception {
        try {
            List<CrossLaneSnapshotDataDTO> list = (List<CrossLaneSnapshotDataDTO>) convertData;
            for (CrossLaneSnapshotDataDTO crossLaneSnapshotDataDTO : list) {
                String json = JacksonUtils.getInstance().writeValueAsString(crossLaneSnapshotDataDTO);
                long timeStamp = crossLaneSnapshotDataDTO.getTimeStamp() * 1000L;
                Date date = new Date(timeStamp);
                String currentDateStr = new SimpleDateFormat("yyyyMMdd").format(date);
                GetIndexRequest getIndexRequest = new GetIndexRequest(Constants.LANE_SNAPSHOT_DATA_ES_INDEX);
                boolean exists = client.indices().exists(getIndexRequest, RequestOptions.DEFAULT);
                if (exists) {
                    IndexRequest indexRequest = new IndexRequest(Constants.LANE_SNAPSHOT_DATA_ES_INDEX);
                    indexRequest.source(json, XContentType.JSON);
                    IndexResponse response = client.index(indexRequest, RequestOptions.DEFAULT);
                } else {
                    CreateIndexRequest createIndexRequest = new CreateIndexRequest(Constants.LANE_SNAPSHOT_DATA_ES_INDEX);
                    createIndexRequest.source(json, XContentType.JSON);
                    CreateIndexResponse createIndexResponse = client.indices().create(createIndexRequest, RequestOptions.DEFAULT);
                }
            }
        } catch (Exception e) {
            log.error("车道快照数据存储elastic异常", e);
            throw new Exception(e);
        }
    }
}
